from . Token import TokenService
from . Application import ApplicationService
from . Airtime import AirtimeService
from . SMS import SMSService
from . Payment import PaymentService
from . Voice import VoiceService

SMS = None
Airtime = None
Payment = None
USSD = None
Voice = None
Application = None
Token = None


def initialize(username, api_key):

    if username is None or api_key is None:
        raise RuntimeError('Invalid username and/or api_key')

    globals()['SMS'] = SMSService(username, api_key)
    globals()['Airtime'] = AirtimeService(username, api_key)
    globals()['Payment'] = PaymentService(username, api_key)
    globals()['Voice'] = VoiceService(username, api_key)
    globals()['Application'] = ApplicationService(username, api_key)
    globals()['Token'] = TokenService(username, api_key)
    # globals()['USSD'] = USSDService(username, api_key)




from . Service import Service, validate_phone


class VoiceService(Service):
    def __init__(self, username, api_key):
        super(VoiceService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = 'https://voice.'
        if self._is_sandbox():
            self._baseUrl += self._SANDBOX_DOMAIN
        else:
            self._baseUrl += self._PRODUCTION_DOMAIN

    def call(self, source, destination, callback=None):

        if not validate_phone(destination):
            raise ValueError('Invalid destination phone number')

        url = self._make_url('/call')
        data = {
            'username': self._username,
            'from': source,
            'to': destination,
        }
        return self._make_request(url, 'POST', headers=self._headers, params=None, data=data, callback=callback)

    def fetch_queued_calls(self, phone_number, callback=None):

        if not validate_phone(phone_number):
            raise ValueError('Invalid phone number')

        url = self._make_url('/queueStatus')
        data = {
            'username': self._username,
            'phoneNumbers': phone_number,
        }
        return self._make_request(url, 'POST', headers=self._headers, params=None, data=data, callback=callback)

    def media_upload(self, phone_number, url, callback=None):

        if not validate_phone(phone_number):
            raise ValueError('Invalid phone number')

        call_url = self._make_url('/mediaUpload')
        data = {
            'username': self._username,
            'phoneNumber': phone_number,
            'url': url,
        }
        return self._make_request(call_url, 'POST', headers=self._headers, params=None, data=data, callback=callback)



"""
Airtime

send(phoneNumber: String, amount: String): Send airtime to a phone number.

send(recipients: Map<String,String>): Send airtime to a bunch of phone numbers. The keys in the recipients map are phone
    numbers while the values are airtime amounts.
"""
import africastalking
import unittest
import random
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Airtime


class TestAirtimeService(unittest.TestCase):

    # def test_send_single(self):
    #     currency_code = "KES"
    #     amount = str(random.randint(10, 1000))
    #     phone = '+25471876' + str(random.randint(1000, 9999))
    #     res = service.send(phone_number=phone, amount=amount, currency_code=currency_code)
    #     assert res['numSent'] == 1

    def test_send_multiple(self):
        res = service.send(recipients=[
            {'phoneNumber': '+2348160663047', 'amount': str(random.randint(1, 10)), 'currency_code': 'USD'},
            {'phoneNumber': '+254718769881', 'amount':str(random.randint(138, 13223)), 'currency_code':  'KES'},
        ])
        assert res['numSent'] == 2


if __name__ == '__main__':
    unittest.main()



import json

from . Service import APIService, validate_phone


class TokenService(APIService):

    def __init__(self, username, api_key):
        super(TokenService, self).__init__(username, api_key)

    def create_checkout_token(self, phone_number, callback=None):

        if not validate_phone(phone_number):
            raise ValueError('Invalid phone number')

        url = self._make_url('/checkout/token/create')
        headers = dict(self._headers)
        data = {'phoneNumber': phone_number}

        return self._make_request(url, 'POST', headers, data=data, params=None, callback=callback)

    def generate_auth_token(self, callback=None):
        
        url = self._make_url('/auth-token/generate')
        headers = dict(self._headers)
        headers['Content-Type'] = 'application/json'
        data = json.dumps({
            'username': self._username
        })

        return self._make_request(url, 'POST', headers, params=None, data=data, callback=callback)





"""
Voice

call(phoneNumber: String): Initiate a phone call

fetchQueuedCalls(phoneNumber: String): Get queued calls

uploadMediaFile(phoneNumber: String, url: String): Upload voice media file

ActionBuilder: Build voice xml when callback URL receives a POST from the voice API
    say(text: String)

    play(url: String)

    getDigits(text: String, url: String, numDigits: Integer, timeout: Integer, finishOnKey: String, callbackUrl: String)

    dial(phoneNumbers: String, ringbackTone: String, record: Boolean, sequential: Boolean, callerId: String, maxDuration: Integer)

    conference()

    record()

    record(String text, URL url, int maxLength, long timeout, String finishOnKey, boolean trimSilence, boolean playBeep, URL callbackUrl)

    enqueue()

    dequeue()

    reject()

    redirect()

    build(): Finally build the xml

"""
import africastalking
import unittest
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Voice


class TestVoiceService(unittest.TestCase):

    def test_call(self):
        res = service.call(source="+254711223355", destination="+254711223366")
        assert res['errorMessage'] != 'None'

    def test_fetch_queued_calls(self):
        res = service.fetch_queued_calls(phone_number='+254711223366')
        
        assert res['status'] == 'Success'

    def test_media_upload(self):
        res = service.media_upload(phone_number='+254711223355', url='https://aksalj.com')
        assert res == 'The request has been fulfilled and resulted in a new resource being created.'


if __name__ == '__main__':
    unittest.main()



from schema import Schema, And, SchemaError
import json
from . Service import APIService, validate_amount, validate_phone


class AirtimeService(APIService):
    def __init__(self, username, api_key):
        super(AirtimeService, self).__init__(username, api_key)

    def _init_service(self):
        super(AirtimeService, self)._init_service()
        self._baseUrl = self._baseUrl + '/version1/airtime'

    def send(self, phone_number=None, amount=None, currency_code=None, recipients=None, callback=None):

        if recipients is not None:
            def join_amount_and_currency(obj):
                obj['amount'] = " ".join([str(obj['currency_code']), str(obj['amount'])])
                del obj['currency_code']
                return obj
            recipients = list(map(join_amount_and_currency, recipients))
        if all(key is not None for key in [phone_number, amount, currency_code ]) and recipients is None:
            amount = " ".join([str(currency_code), str(amount)])
            recipients = [
                {'phoneNumber': str(phone_number), 'amount': str(amount)},
            ]

        try:
            schema = Schema([
                {
                    'phoneNumber': And(str, lambda s: validate_phone(s)),
                    'amount': And(str, lambda s: validate_amount(s))
                }
            ])
            recipients = schema.validate(recipients)
        except SchemaError as err:
            raise ValueError('Invalid recipients: ' + err.message)

        url = self._make_url('/send')
        data = {
            'username': self._username,
            'recipients': json.dumps(recipients)
        }
        return self._make_request(url, 'POST', headers=self._headers, params=None, data=data, callback=callback)



from . Service import APIService, validate_phone


class SMSService(APIService):
    def __init__(self, username, api_key):
        super(SMSService, self).__init__(username, api_key)

    def _init_service(self):
        super(SMSService, self)._init_service()
        self._baseUrl = self._baseUrl + '/version1'

    def send(self, message, recipients, sender_id=None, enqueue=False, callback=None):

        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError('Invalid phone number: ' + phone)

        url = self._make_url('/messaging')
        data = {
            'username': self._username,
            'to': ','.join(recipients),
            'message': message,
            'bulkSMSMode': 1,
        }

        if sender_id is not None:
            data['from'] = sender_id

        if enqueue:
            data['enqueue'] = 1

        return self._make_request(url, 'POST', headers=self._headers, params=None, data=data, callback=callback)

    def send_premium(self, message, keyword, link_id, recipients, sender_id=None,
                     retry_duration_in_hours=None, callback=None):

        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError('Invalid phone number: ' + phone)

        url = self._make_url('/messaging')
        data = {
            'username': self._username,
            'to': ','.join(recipients),
            'message': message,
            'bulkSMSMode': 0,
            'keyword': keyword,
            'linkId': link_id
        }

        if sender_id is not None:
            data['from'] = sender_id

        if retry_duration_in_hours is not None:
            data['retryDurationInHours'] = retry_duration_in_hours

        return self._make_request(url, 'POST', headers=self._headers, params=None, data=data, callback=callback)

    def fetch_messages(self, last_received_id=None, callback=None):
        url = self._make_url('/messaging')
        params = {
            'username': self._username
        }

        if last_received_id is not None:
            params['lastReceivedId'] = last_received_id

        return self._make_request(url, 'GET', headers=self._headers, params=params, data=None, callback=callback)

    def fetch_subscriptions(self, short_code, keyword, last_received_id=None, callback=None):
        url = self._make_url('/subscription')
        params = {
            'username': self._username,
            'shortCode': short_code,
            'keyword': keyword
        }

        if last_received_id is not None:
            params['lastReceivedId'] = last_received_id

        return self._make_request(url, 'GET', headers=self._headers, params=params, data=None, callback=callback)

    def create_subscription(self, short_code, keyword, phone_number, checkout_token, callback=None):

        if not validate_phone(phone_number):
            raise ValueError('Invalid phone number')

        url = self._make_url('/subscription/create')
        data = {
            'username': self._username,
            'shortCode': short_code,
            'keyword': keyword,
            'phoneNumber': phone_number,
            'checkoutToken': checkout_token,
        }

        return self._make_request(url, 'POST', headers=self._headers, data=data, params=None, callback=callback)
    
    def delete_subscription(self, short_code, keyword, phone_number, callback=None):

        if not validate_phone(phone_number):
            raise ValueError('Invalid phone number')

        url = self._make_url('/subscription/delete')
        data = {
            'username': self._username,
            'shortCode': short_code,
            'keyword': keyword,
            'phoneNumber': phone_number
        }

        return self._make_request(url, 'POST', headers=self._headers, data=data, params=None, callback=callback)

